# -*- coding:utf-8 -*-
from .winConst import *
from .myctypes import *
import binascii
class NetWork():
    def __init__(self,miniblink):     
        self.types=['.jpg','.png','.mp4','.ts','.mp3','.avi','.gif']
        self.bufs=[]
        self.mb=miniblink
    def wkeLoadURLW(self,webview,url):
        self.mb.wkeLoadURLW(webview,url)
    def wkeLoadHTMLW(self,webview,html):
        self.mb.wkeLoadHTMLW(webview,html)
    def wkeLoadFile(self,webview,file_path):
        file_path=file_path.encode()
        self.mb.wkeLoadFile(webview,file_path)
    def wkePostURLW(self,webview,url,data):
        data=data.encode()
        lens=len(data)
        self.mb.wkeLoadURLW(webview,data,lens)
    def wkeReload(self,webview):
        self.mb.wkeReload(webview)
    def wkeStopLoading(self,webview):
        self.mb.wkeStopLoading(webview)
    def wkeGetURL(self,webview):
        url=self.mb.wkeGetURL(webview)
        return url.decode()
    def wkeGetFrameUrl(self,webview,frameId):
        url=self.mb.wkeGetFrameUrl(webview,frameId)
        return url.decode('utf8')  
    def wkeGetSource(self,webview):
        source=self.mb.wkeGetSource(webview)
        return source.decode()
    def wkeUtilSerializeToMHTML(self,webview):
        mhtml_content=self.mb.wkeUtilSerializeToMHTML(webview)
 
        return mhtml_content
    def get_type(self,url):
        for x in self.types:
            if x in url:
                return x
        return    
 
    def save_buf_data(self,url,buf,lens):
   
        if lens==0:return
        contents=(c_char * lens).from_address(buf)
        _type=self.get_type(url)
        if _type==None:return
        name=binascii.crc32(url.encode())
        file_name=f'{name}{_type}'
        try:
            with open(file_name,'wb') as f:
                f.write(contents)
            self.bufs.append({url:file_name})
        except:
            ...
        finally:
            ...
 
    def cancel_request(self,job,url,ident_ls=['.jpg']):

        for x in ident_ls:
            if  x in url:
                self.mb.wkeNetCancelRequest(job)
                return True
        return False
    def set_response_data(self,job,data='',file_name=None):

        lens=len(data)
        if lens!=0:
            self.mb.wkeNetSetData(job,data,lens)
            return True
        elif file_name!=None:
            with open(file_name) as f:
                data=f.read()
                data=data.encode()
                lens=len(data)
            if lens!=0:
                if '.js' in file_name:
                    self.mb.wkeNetSetMIMEType(job,b'text/javascript')
                elif '.html' in file_name:
                    self.mb.wkeNetSetMIMEType(job,b'text/html')
                self.mb.wkeNetSetData(job,data,lens)
                return True
        return False    
   
    def get_post_data(self,job,url,ident=''):
        
        if ident not in url:return '',0,None
        elements=self.mb.wkeNetGetPostBody(job)
        try:
            data=elements.contents.element.contents.contents.data.contents.data
            lens=elements.contents.element.contents.contents.data.contents.length
        except:
            return '',0,None
        data=data[:lens].decode('utf8','ignore')
        print('post_data',data,lens)
        return data,lens,elements


# -*- coding:utf-8 -*-
from ctypes import (
    c_int,
    c_ushort,
    c_long,
    c_longlong,
    c_char,
    c_char_p,
    c_wchar_p,
    c_bool,
    c_void_p,
    c_size_t,
    Structure,
    POINTER
    )

from ctypes.wintypes import (
    DWORD,
    LONG,
    WORD,
    BYTE
)

class wkeProxy(Structure):

    _fields_ = [('type', c_int),('hostname', c_char *100),('port', c_ushort ),('username', c_char *50),('password',c_char *50)]
class wkeRect(Structure):

    _fields_=[('x',c_int),('y',c_int),('w',c_int),('h',c_int)]
class wkeMemBuf(Structure):

    _fields_=[('size',c_int),('data',c_char_p),('length',c_size_t)]
class wkeString(Structure):
    ...
class wkePostBodyElement(Structure):

    _fields_=[('size',c_int),('type',c_int),('data',POINTER(wkeMemBuf)),('filePath',wkeString),('fileStart',c_longlong),('fileLength',c_longlong)]
    ...
class wkePostBodyElements(Structure):

    _fields_ =[('size',c_int),('element',POINTER(POINTER(wkePostBodyElement))),('elementSize',c_size_t),('isDirty',c_bool)]
class wkeScreenshotSettings(Structure):

    _fields_=[('structSize',c_int),('width',c_int),('height',c_int)]
class wkeWindowFeatures(Structure):

    _fields_=[('x',c_int),('y',c_int),('width',c_int),('height',c_int),('menuBarVisible',c_bool),('statusBarVisible',c_bool),('toolBarVisible',c_bool),('locationBarVisible',c_bool),('scrollbarsVisible',c_bool),('resizable',c_bool),('fullscreen',c_bool)]

class wkePrintSettings(Structure):

    _fields_=[('structSize',c_int),('dpi',c_int),('width',c_int),('height',c_int),('marginTop',c_int),('marginBottom',c_int),('marginLeft',c_int),('marginRight',c_int),('isPrintPageHeadAndFooter',c_bool),('isPrintBackgroud',c_bool),('isLandscape',c_bool)]
class wkePdfDatas(Structure):

    _fields_=[('count',c_int),('sizes',c_size_t),('datas',c_void_p)]

class Rect(Structure):

    _fields_=[('Left',c_int),('Top',c_int),('Right',c_int),('Bottom',c_int)]

class mPos(Structure):

    _fields_=[('x',c_int),('y',c_int)]

class mSize(Structure):
    ...
mSize._fields_=[('cx',c_int),('cy',c_int)]

class bitMap(Structure):

    _fields_=[('bmType',c_int),('bmWidth',c_int),('bmHeight',c_int),('bmWidthBytes',c_int),('bmPlanes',c_int),('bmBitsPixel',c_int),('bmBits',c_int)]

class blendFunction(Structure):

    _fields_=[('BlendOp',BYTE),('BlendFlags',BYTE),('SourceConstantAlpha',BYTE),('AlphaFormat',BYTE)]


class COMPOSITIONFORM(Structure):

    _fields_=[('dwStyle',c_int),('ptCurrentPos',mPos),('rcArea',Rect)]


class BITMAPINFOHEADER(Structure):
    """ 关于DIB的尺寸和颜色格式的信息 """
    _fields_ = [
        ("biSize", DWORD),
        ("biWidth", LONG),
        ("biHeight", LONG),
        ("biPlanes", WORD),#永远为1
        ("biBitCount", WORD),#1(双色)，4(16色)，8(256色)，24(真彩色)，32(真彩色)
        ("biCompression", DWORD),#0不压缩
        ("biSizeImage", DWORD),#表示位图数据的大小以字节为单位
        ("biXPelsPerMeter", LONG),
        ("biYPelsPerMeter", LONG),
        ("biClrUsed", DWORD),#位图实际使用的颜色表中的颜色数
        ("biClrImportant", DWORD)#位图显示过程中重要的颜色数
    ]
class BITMAPFILEHEADER(Structure):
    __file__=[
        ('bfType',c_int),#BMP类型：19778，也就是BM
        ('bfSize',c_int),#文件字节数：14 + BITMAPINFOHEADER.biSize + BITMAPINFOHEADER.biSizeImage
        ('bfReserved1',c_int),
        ('bfReserved2',c_int),
        ('bfOffBits',c_int)#位图的数据信息离文件头的偏移量:14 + BITMAPINFOHEADER.biSize
    ]
class BITMAPINFO(Structure):

    _fields_ = [("bmiHeader", BITMAPINFOHEADER), ("bmiColors", DWORD * 3)]


from config import _LRESULT
class PAINTSTRUCT(Structure):
    _fields_=[('hdc',_LRESULT),('fErase',c_int),('rcPaint',Rect),('fRestore',c_int),('fIncUpdate',c_int),('rgbReserved',c_char *32)]



# -*- coding:utf-8 -*-

from .winConst import *
from .myctypes import *
from .wkeStruct import (wkeProxy,wkePostBodyElements,wkeRect)

from .bindwebview import BindWebview
from .callback import CallBack
from .cookie import Cookie
from .jsrunpy import JsRunPy
from .message import Message
from .network import NetWork
from .proxy import Proxy
from .pyrunjs import PyRunJS

import platform
from .window import Window
from config import node_path,_LRESULT

class MiniBlink():
    def init():

        architecture=platform.architecture()[0]
        if architecture=='64bit' and (not node_path.endswith('x64.dll')):
            print('请使用与node.dll位数对应的Python版本')
            return 0
        elif architecture=='32bit' and node_path.endswith('x64.dll'):
            print('请使用与node.dll位数对应的Python版本')
            return 0

        mb=cdll.LoadLibrary(node_path)
        mb.wkeInit()
        mb.wkeCreateWebWindow.argtypes=[_LRESULT]
        mb.wkeCreateWebWindow.restype=_LRESULT
        mb.wkeCreateWebView.restype=_LRESULT        
        mb.wkeSetWindowTitleW.argtypes=[_LRESULT]
        mb.wkeSetTransparent.argtypes=[_LRESULT]
        mb.wkeSetHandleOffset.argtypes=[_LRESULT]
        mb.wkeSetHandle.argtypes=[_LRESULT]
        mb.wkeKillFocus.argtypes=[_LRESULT]
        mb.wkeRepaintIfNeeded.argtypes=[_LRESULT]
        mb.wkeWake.argtypes=[_LRESULT]
        mb.wkeGetCaretRect.argtypes=[_LRESULT]
        mb.wkeGetCaretRect.restype=wkeRect
        mb.wkeResize.argtypes=[_LRESULT]
        mb.wkeShowWindow.argtypes=[_LRESULT]
        mb.wkeMoveToCenter.argtypes=[_LRESULT]
        mb.wkeGoForward.argtypes=[_LRESULT]
        mb.wkeGoBack.argtypes=[_LRESULT]
        mb.wkeLoadURLW.argtypes=[_LRESULT]
        mb.wkeLoadHTMLW.argtypes=[_LRESULT]
        mb.wkeLoadFile.argtypes=[_LRESULT]
        mb.wkeReload.argtypes=[_LRESULT]
        mb.wkeStopLoading.argtypes=[_LRESULT]
        mb.wkeWidth.argtypes=[_LRESULT]
        mb.wkeWidth.restype=_LRESULT
        mb.wkeHeight.argtypes=[_LRESULT]
        mb.wkeHeight.restype=_LRESULT
        mb.wkeContentsWidth.argtypes=[_LRESULT]
        mb.wkeContentsWidth.restype=_LRESULT
        mb.wkeContentsHeight.argtypes=[_LRESULT]
        mb.wkeContentsHeight.restype=_LRESULT
        mb.wkeGetWindowHandle.argtypes=[_LRESULT]
        mb.wkeGetWindowHandle.restype=_LRESULT
        mb.wkeGetURL.argtypes=[_LRESULT]
        mb.wkeGetURL.restype=c_char_p
        mb.wkeGetFrameUrl.argtypes=[_LRESULT]
        mb.wkeGetFrameUrl.restype=c_char_p
        mb.wkeGetSource.argtypes=[_LRESULT]
        mb.wkeGetSource.restype=c_char_p
        mb.wkeUtilSerializeToMHTML.argtypes=[_LRESULT]
        mb.wkeUtilSerializeToMHTML.restype=c_char_p
        mb.wkeGetViewDC.argtypes=[_LRESULT]
        mb.wkeGetViewDC.restype=_LRESULT
        mb.wkeFireMouseEvent.argtypes=[_LRESULT]
        mb.wkeFireKeyDownEvent.argtypes=[_LRESULT]
        mb.wkeFireKeyUpEvent.argtypes=[_LRESULT]
        mb.wkeFireKeyPressEvent.argtypes=[_LRESULT]
        mb.wkeFireWindowsMessage.argtypes=[_LRESULT]
        mb.wkeFireMouseWheelEvent.argtypes=[_LRESULT,c_int,c_int,_LRESULT,c_int]
        mb.wkeFireContextMenuEvent.argtypes=[_LRESULT]

        mb.wkeOnCreateView.argtypes=[_LRESULT]
        mb.wkeOnPaintUpdated.argtypes=[_LRESULT]
        mb.wkeOnPaintBitUpdated.argtypes=[_LRESULT]
        mb.wkeOnNavigation.argtypes=[_LRESULT]
        mb.wkeOnTitleChanged.argtypes=[_LRESULT]
        mb.wkeOnURLChanged2.argtypes=[_LRESULT]
        mb.wkeOnMouseOverUrlChanged.argtypes=[_LRESULT]
        mb.wkeOnAlertBox.argtypes=[_LRESULT]
        mb.wkeOnConfirmBox.argtypes=[_LRESULT]
        mb.wkeOnPromptBox.argtypes=[_LRESULT]
        mb.wkeOnConsole.argtypes=[_LRESULT]
        mb.wkeOnDownload.argtypes=[_LRESULT]
        mb.wkeOnDocumentReady2.argtypes=[_LRESULT]
        mb.wkeNetOnResponse.argtypes=[_LRESULT]
        mb.wkeOnLoadUrlBegin.argtypes=[_LRESULT]
        mb.wkeOnLoadUrlEnd.argtypes=[_LRESULT]
        mb.wkeOnLoadUrlEnd.argtypes=[_LRESULT]
        mb.wkeOnLoadingFinish.argtypes=[_LRESULT]
        mb.wkeOnLoadUrlFail.argtypes=[_LRESULT]
        mb.wkeNetGetFavicon.argtypes=[_LRESULT]
        mb.wkeOnWindowClosing.argtypes=[_LRESULT]
        mb.wkeOnWindowDestroy.argtypes=[_LRESULT]

        mb.wkeIsDocumentReady.argtypes=[_LRESULT]
        mb.wkeNetHookRequest.argtypes=[_LRESULT]
        mb.wkeNetGetRequestMethod.argtypes=[_LRESULT]
        mb.wkeNetGetRequestMethod.restype=_LRESULT
        mb.jsArgCount.argtypes=[_LRESULT]
        mb.jsArgCount.restype=_LRESULT
        mb.wkeGlobalExec.argtypes=[_LRESULT]
        mb.wkeGlobalExec.restype=_LRESULT
        mb.jsGetGlobal.argtypes=[_LRESULT,c_char_p]
        mb.jsGetGlobal.restype=_LRESULT
        mb.jsGet.argtypes=[_LRESULT]
        mb.jsGet.restype=_LRESULT
        mb.wkeRunJSW.argtypes=[_LRESULT,c_wchar_p]
        mb.wkeRunJSW.restype=c_longlong
        mb.jsToStringW.argtypes=[_LRESULT,c_longlong]
        mb.jsToStringW.restype=c_wchar_p
        mb.wkeRunJsByFrame.argtypes=[_LRESULT]
        mb.wkeRunJsByFrame.restype=_LRESULT
        mb.wkeGetGlobalExecByFrame.argtypes=[_LRESULT]
        mb.wkeGetGlobalExecByFrame.restype=_LRESULT
        mb.wkeJsBindFunction.argtypes=[c_char_p]
        mb.jsToTempStringW.argtypes=[_LRESULT]
        mb.jsToTempStringW.restype=c_wchar_p
        mb.jsArgType.argtypes=[_LRESULT]
        mb.jsArgType.restype=_LRESULT
        mb.jsArg.argtypes=[_LRESULT]
        mb.jsArg.restype=_LRESULT
        mb.jsGetLength.argtypes=[_LRESULT]
        mb.jsGetLength.restype=_LRESULT
        mb.jsGetAt.argtypes=[_LRESULT]
        mb.jsGetAt.restype=_LRESULT
        mb.jsSetAt.argtypes=[_LRESULT]
        mb.jsGetKeys.argtypes=[_LRESULT]
  
        mb.jsCall.argtypes=[_LRESULT]
        mb.jsCall.restype=_LRESULT
        mb.jsIsNumber.argtypes=[c_longlong]
        mb.jsIsNumber.restype=_LRESULT
        mb.jsToInt.argtypes=[_LRESULT]
        mb.jsToInt.restype=_LRESULT
        mb.jsIsString.argtypes=[c_longlong]
        mb.jsIsString.restype=_LRESULT
        mb.jsIsBoolean.argtypes=[c_longlong]
        mb.jsIsBoolean.restype=_LRESULT
        mb.jsStringW.argtypes=[_LRESULT,c_wchar_p]
        mb.jsEmptyArray.argtypes=[_LRESULT]
        mb.jsEmptyArray.restype=_LRESULT
        mb.jsStringW.restype=_LRESULT
        mb.jsBoolean.argtypes=[c_bool]
        mb.jsBoolean.restype=_LRESULT
        mb.jsFloat.argtypes=[c_float]
        mb.jsInt.argtypes=[_LRESULT]
        mb.jsInt.restype=_LRESULT
        mb.jsEmptyObject.argtypes=[_LRESULT]
        mb.jsEmptyObject.restype=_LRESULT
        mb.jsSet.argtypes=[_LRESULT]
        mb.wkeSetLocalStorageFullPath.argtypes=[_LRESULT,c_wchar_p]
        mb.wkeSetCookieEnabled.argtypes=[_LRESULT]
        mb.wkeSetCookie.argtypes=[_LRESULT]
        mb.wkePerformCookieCommand.argtypes=[_LRESULT]
        mb.wkeSetCookieJarPath.argtypes=[_LRESULT,c_wchar_p]
        mb.wkeSetCookieJarFullPath.argtypes=[_LRESULT,c_wchar_p]
        mb.wkeClearCookie.argtypes=[_LRESULT]
        mb.wkeSetProxy.argtypes=[POINTER(wkeProxy)]
        mb.wkeSetViewProxy.argtypes=[_LRESULT,POINTER(wkeProxy)]
        mb.wkeNetGetPostBody.argtypes=[_LRESULT]
        mb.wkeNetGetPostBody.restype=POINTER(wkePostBodyElements)
        mb.wkeNetCancelRequest.argtypes=[_LRESULT]
        mb.wkeNetSetData.argtypes=[_LRESULT,c_char_p]
        mb.wkeNetSetMIMEType.argtypes=[_LRESULT,c_char_p]
        mb.wkePostURLW.argtypes=[_LRESULT,c_wchar_p,c_char_p]
        mb.wkeSetTouchEnabled.argtypes=[_LRESULT]
        mb.wkeSetDeviceParameter.argtypes=[_LRESULT]
        mb.wkeSetWebViewName.argtypes=[_LRESULT]
        mb.wkeSetZoomFactor.argtypes=[_LRESULT]
        mb.wkeSetNavigationToNewWindowEnable.argtypes=[_LRESULT]
        mb.wkeSetContextMenuEnabled.argtypes=[_LRESULT]
        mb.wkeSetHeadlessEnabled.argtypes=[_LRESULT]
        mb.wkeSetDragEnable.argtypes=[_LRESULT]
        mb.wkeAddPluginDirectory.argtypes=[_LRESULT,c_wchar_p]
        mb.wkeSetNpapiPluginsEnabled.argtypes=[_LRESULT]
        mb.wkeSetCspCheckEnable.argtypes=[_LRESULT]
        mb.wkeSetDebugConfig.argtypes=[_LRESULT]
        mb.wkeSetString.argtypes=[_LRESULT]
        mb.wkeSetUserAgentW.argtypes=[_LRESULT]
        mb.wkeGetUserAgent.argtypes=[_LRESULT]
        mb.wkeGetUserAgent.restype=c_char_p

        mb.wkeGetCookieW.argtypes=[_LRESULT]
        mb.wkeGetCookieW.restype=c_wchar_p
        mb.wkeGetStringW.argtypes=[_LRESULT]
        mb.wkeGetStringW.restype=c_wchar_p

        MiniBlink.bindwebview=BindWebview(mb)
        MiniBlink.callback=CallBack(mb)
        MiniBlink.cookie=Cookie(mb)
        MiniBlink.jsrunpy=JsRunPy(mb)
        MiniBlink.message=Message(mb)
        MiniBlink.network=NetWork(mb)
        MiniBlink.proxy=Proxy(mb)
        MiniBlink.pyrunjs=PyRunJS(mb)
        MiniBlink.window=Window(mb)    
        
        return mb




# -*- coding:utf-8 -*-
from .winConst import *
from .myctypes import *
from .wkeStruct import (Rect,mPos,mSize,COMPOSITIONFORM,bitMap,blendFunction,PAINTSTRUCT)
from .method import method
from .callback import CallBack
from win32gui import SetWindowLong
from config import _LRESULT

user32.CallWindowProcW.argtypes=[_LRESULT,HWND, UINT,WPARAM,LPARAM]
user32.CallWindowProcW.restype=_LRESULT
class BindWebview():
    def __init__(self,miniblink,webview=0):
        global js
        self.mb=miniblink
        self.m_webview=webview
        self.cb=CallBack(miniblink)
        self.cb.wkePaintUpdatedCallback=self.__paint_func
    def bind_webview(self,hwnd=0,isTransparent=False,isZoom=True):
        
        if user32.IsWindow(hwnd)==0:
            return 0
        if self.m_webview==0:
           self.m_webview=self.mb.wkeCreateWebView()
        if self.m_webview==0:
            return 0
        self.isZoom=isZoom
        self.mb.wkeSetHandle(self.m_webview,hwnd)
        self.cb.wkeOnPaintUpdated(self.m_webview,hwnd)

        if isTransparent:
            self.mb.wkeSetTransparent(self.m_webview,1)
            exStyle=user32.GetWindowLongW(hwnd,WinConst.GWL_EXSTYLE)
            SetWindowLong(hwnd,WinConst.GWL_EXSTYLE,exStyle | WinConst.WS_EX_LAYERED)
        else:
            self.mb.wkeSetTransparent(self.m_webview,0)
        self.__onMyWndProc(hwnd)
        rc=Rect()
        user32.GetClientRect(hwnd,byref(rc))
        self.mb.wkeResize(self.m_webview,rc.Right - rc.Left, rc.Bottom - rc.Top)
        return self.m_webview
    def __paint_func(self,**kwargs):
        webview=kwargs['webview']
        param=kwargs['param']
        hdc=kwargs['hdc']
        x=kwargs['x']
        y=kwargs['y']
        cx=kwargs['cx']
        cy=kwargs['cy']

        if param==None:return
        hwnd=param
        if (user32.GetWindowLongW(hwnd,WinConst.GWL_EXSTYLE) & WinConst.WS_EX_LAYERED)== WinConst.WS_EX_LAYERED:

            self.__transparentPaint(hwnd, hdc, x, y, cx, cy)
        else:
            rc=Rect(0,0,x+cx,y+cy)
   
            user32.InvalidateRect(hwnd, byref(rc), True)
    def __onMyWndProc(self,hwnd):
        self.oldWndProc=SetWindowLong(hwnd,WinConst.GWL_WNDPROC,self.__myWndProcCallBack)
    @method(WINFUNCTYPE(_LRESULT,HWND, UINT,WPARAM,LPARAM))
    def __myWndProcCallBack(self,hwnd,msg,wParam,lParam):
        if msg==WinConst.WM_PAINT:
   
            if WinConst.WS_EX_LAYERED!=(WinConst.WS_EX_LAYERED & user32.GetWindowLongW(hwnd,WinConst.GWL_EXSTYLE)):
                ps=PAINTSTRUCT()
                hdc=user32.BeginPaint(hwnd,byref(ps))
                rcClip = ps.rcPaint
                rcClient=Rect()
                user32.GetClientRect(hwnd,byref(rcClient))

                rcInvalid=rcClient
                if (rcClient.Right != rcClip.Left) and (rcClip.Bottom != rcClip.Top):
                    user32.IntersectRect(byref(rcInvalid),byref(rcClip),byref(rcClient))
                    srcX = rcInvalid.Left - rcClient.Left
                    srcY = rcInvalid.Top - rcClient.Top
                    destX = rcInvalid.Left
                    destY = rcInvalid.Top
                    width = rcInvalid.Right - rcInvalid.Left
                    height = rcInvalid.Bottom - rcInvalid.Top
                    if width!=0 and height!=0:
                      
                        tmp_dc=self.mb.wkeGetViewDC(self.m_webview)
                        gdi32.BitBlt(hdc,destX, destY, width, height,tmp_dc,srcX, srcY,WinConst.SRCCOPY)
                        user32.EndPaint(hwnd,byref(ps))
                    return 0
        elif msg==WinConst.WM_ERASEBKGND:
            return 1
        elif msg==WinConst.WM_SIZE:
            width= lParam & 65535
            height= lParam >> 16
            self.mb.wkeResize(self.m_webview,width,height)
            return 0
        elif msg==WinConst.WM_KEYDOWN:
            virtualKeyCode=wParam
            flags=0
            if ((lParam >> 16) & WinConst.KF_REPEAT)!=0:
                flags=flags | 0x4000
            if ((lParam >> 16)  & WinConst.KF_EXTENDED)!=0:
                flags=flags | 0x0100
            if self.mb.wkeFireKeyDownEvent(self.m_webview,virtualKeyCode,flags)!=0:
                return 0
        elif msg==WinConst.WM_KEYUP:
            virtualKeyCode=wParam
            flags=0
            if virtualKeyCode==116:
                self.mb.wkeReload(self.m_webview)
            if ((lParam >> 16) & WinConst.KF_REPEAT)!=0:
                flags=flags | 0x4000
            if ((lParam >> 16) & WinConst.KF_EXTENDED)!=0:
                flags=flags | 0x0100
            if self.mb.wkeFireKeyUpEvent(self.m_webview,virtualKeyCode,flags)!=0:
                return 0
        elif msg==WinConst.WM_CHAR:
            virtualKeyCode=wParam
            flags=0
            if ((lParam >> 16) & WinConst.KF_REPEAT)!=0:
                flags=flags | 0x4000
            if self.mb.wkeFireKeyPressEvent(self.m_webview,virtualKeyCode,flags)!=0:
                return 0
        elif msg in [WinConst.WM_LBUTTONDOWN,WinConst.WM_MBUTTONDOWN,WinConst.WM_RBUTTONDOWN,WinConst.WM_LBUTTONDBLCLK,WinConst.WM_MBUTTONDBLCLK,WinConst.WM_RBUTTONDBLCLK,WinConst.WM_LBUTTONUP,WinConst.WM_MBUTTONUP,WinConst.WM_RBUTTONUP]:
            x=lParam & 65535
            y=lParam >> 16
            flags=0
            if msg in [WinConst.WM_LBUTTONDOWN,WinConst.WM_MBUTTONDOWN,WinConst.WM_RBUTTONDOWN]:
                if user32.GetFocus()!=hwnd:
                    user32.SetFocus(hwnd)
                user32.SetCapture(hwnd)
            elif msg in [WinConst.WM_LBUTTONUP,WinConst.WM_MBUTTONUP,WinConst.WM_RBUTTONUP]:
                user32.ReleaseCapture()
            if (wParam & WinConst.MK_CONTROL)!=0:
                flags=flags | 8
            elif (wParam & WinConst.MK_SHIFT)!=0:
                flags=flags | 4
            elif (wParam & WinConst.MK_LBUTTON)!=0:
                flags=flags | 1
            elif (wParam & WinConst.MK_MBUTTON)!=0:
                flags=flags | 16
            elif (wParam & WinConst.MK_RBUTTON)!=0:
                flags=flags | 2
            self.mb.wkeFireMouseEvent(self.m_webview, msg, x, y, flags)
            return 0

        elif msg==WinConst.WM_MOUSEMOVE:
            x=lParam & 65535
            y=lParam >> 16
            flags=0
            if (wParam & WinConst.MK_LBUTTON)!=0:
                flags=flags | 1
            if self.mb.wkeFireMouseEvent(self.m_webview, msg, x, y, flags)!=0:
                return 0
        elif msg==WinConst.WM_CONTEXTMENU:
            pt=mPos()
            pt.x=lParam & 65535
            pt.y=lParam >> 16
            if pt.x!=-1 and pt.y!=-1:
                user32.ScreenToClient(hwnd,byref(pt))
            flags=0
            if (wParam & WinConst.MK_CONTROL)!=0:
                flags=flags | 8
            if (wParam & WinConst.MK_SHIFT)!=0:
                flags=flags | 4
            if (wParam & WinConst.MK_LBUTTON)!=0:
                flags=flags | 1
            if (wParam & WinConst.MK_MBUTTON)!=0:
                flags=flags | 16
            if (wParam & WinConst.MK_RBUTTON)!=0:
                flags=flags | 2

            if self.mb.wkeFireContextMenuEvent(self.m_webview,pt.x,pt.y, flags)!=0:
                return 0
        elif msg==WinConst.WM_MOUSEWHEEL:
            pt=mPos()
            pt.x=lParam & 65535
            pt.y=lParam >> 16
            user32.ScreenToClient(hwnd,byref(pt))
            delta= wParam >> 16
            flags=0

            if (wParam & WinConst.MK_CONTROL)!=0:
                flags=flags | 8
            if (wParam & WinConst.MK_SHIFT)!=0:
                flags=flags | 4
            if (wParam & WinConst.MK_LBUTTON)!=0:
                flags=flags | 1
            if (wParam & WinConst.MK_MBUTTON)!=0:
                flags=flags | 16
            if (wParam & WinConst.MK_RBUTTON)!=0:
                flags=flags | 2
            if self.mb.wkeFireMouseWheelEvent(self.m_webview,pt.x,pt.y,delta,flags)!=0:
                return 0
        elif msg==WinConst.WM_SETFOCUS:
            user32.SetFocus(hwnd)
            return 0
        elif msg==WinConst.WM_KILLFOCUS:
            self.mb.wkeKillFocus(self.m_webview)
            return 0
        elif msg==WinConst.WM_IME_STARTCOMPOSITION:
            caret=self.mb.wkeGetCaretRect(self.m_webview)
            mposForm=COMPOSITIONFORM()
            mposForm.dwStyle = 2 | 32
            mposForm.ptCurrentPos.x = caret.x
            mposForm.ptCurrentPos.y = caret.y
            hIMC=imm32.ImmGetContext(hwnd)
            imm32.ImmSetCompositionWindow(hIMC,byref(mposForm))
            imm32.ImmReleaseContext(hwnd,hIMC)
            return 0
        elif msg==WinConst.WM_SETCURSOR:
            if self.mb.wkeFireWindowsMessage(self.m_webview,hwnd,WinConst.WM_SETCURSOR,wParam,lParam,0)!=0:
                return 0
        elif msg==WinConst.WM_NCHITTEST:
            if self.isZoom:
                return self.__on_nchittest(hwnd,lParam)
        elif msg==WinConst.WM_INPUTLANGCHANGE:
            return user32.DefWindowProcA(hwnd, msg, _LRESULT(wParam), _LRESULT(lParam))
            
        return user32.CallWindowProcW(self.oldWndProc, hwnd, msg, wParam, lParam)
    def __transparentPaint(self,hwnd,hdc,x,y,cx,cy):
        rectDest=Rect()
        user32.GetClientRect(hwnd,byref(rectDest))
        user32.OffsetRect(byref(rectDest),-rectDest.Left,-rectDest.Top)

        width = rectDest.Right - rectDest.Left
        height = rectDest.Bottom - rectDest.Top
        hBitmap = gdi32.GetCurrentObject(hdc, WinConst.OBJ_BITMAP)

        bmp=bitMap()
        bmp.bmType=0
        cbBuffer=gdi32.GetObjectA(hBitmap, 24,0)
        gdi32.GetObjectA(hBitmap, cbBuffer,byref(bmp))
        sizeDest=mSize()
        sizeDest.cx =bmp.bmWidth
        sizeDest.cy =bmp.bmHeight

        hdcScreen = user32.GetDC(_LRESULT(hwnd))
        blendFunc32bpp=blendFunction()
        blendFunc32bpp.BlendOp = 0   
        blendFunc32bpp.BlendFlags = 0
        blendFunc32bpp.SourceConstantAlpha = 255
        blendFunc32bpp.AlphaFormat = 1  
        pointSource=mPos()
        callOk = user32.UpdateLayeredWindow(hwnd, hdcScreen, 0, byref(sizeDest), hdc, byref(pointSource), RGB(255,255,255), byref(blendFunc32bpp), WinConst.ULW_ALPHA)
 
        user32.ReleaseDC(hwnd, hdcScreen)
    def __on_nchittest(self,hwnd,lParam):
        if user32.IsZoomed(hwnd)!=0:
            return 1
        pt=mPos()
        pt.x=lParam & 65535
        pt.y=lParam >> 16
        user32.ScreenToClient(hwnd,byref(pt))
        rc=Rect()
        user32.GetClientRect(hwnd,byref(rc))
        iWidth = rc.Right - rc.Left
        iHeight = rc.Bottom - rc.Top
        if user32.PtInRect(byref(Rect(5, 0, iWidth - 5, 5)),pt):
            retn=12#HTTOP
        elif user32.PtInRect(byref(Rect(0, 5, 5, iHeight - 5)),pt):
            retn=10#HTLEFT
        elif user32.PtInRect(byref(Rect(iWidth - 5, 5, iWidth, iHeight - 10)),pt):
            retn=11#HTRIGHT
        elif user32.PtInRect(byref(Rect(5, iHeight - 5, iWidth - 10, iHeight)),pt):
            retn=15#HTBOTTOM
        elif user32.PtInRect(byref(Rect(0, 0, 5, 5)),pt):
            retn=13#HTTOPLEFT
        elif user32.PtInRect(byref(Rect(0, iHeight - 5, 5, iHeight)),pt):
            retn=16#HTBOTTOMLEFT
        elif user32.PtInRect(byref(Rect(iWidth - 5, 0, iWidth, 5)),pt):
            retn=14#HTTOPRIGHT
        elif user32.PtInRect(byref(Rect(iWidth - 10, iHeight - 10, iWidth, iHeight)),pt):
            retn=17#HTBOTTOMRIGHT
        else:
            retn=1
        return retn



# -*- coding:utf-8 -*-

from .winConst import *
from .myctypes import *
from .wkeStruct import mPos

class Message():
    def __init__(self,miniblink):

        self.mb=miniblink
    def wkeFireMouseEvent(self,webview,msg,x,y,flags=0):

        return self.mb.wkeFireMouseEvent(webview,msg,x,y,flags)
    def wkeFireKeyDownEvent(self,webview,virtualKeyCode,flags=0):
    
        return self.mb.wkeFireKeyDownEvent(webview,virtualKeyCode,flags,False)
    def wkeFireKeyUpEvent(self,webview,virtualKeyCode,flags=0):

        return self.mb.wkeFireKeyUpEvent(webview,virtualKeyCode,flags,False)
    def wkeFireKeyPressEvent(self,webview,virtualKeyCode,flags):
        return self.mb.wkeFireKeyPressEvent(webview,virtualKeyCode,flags,False)
    def wkeFireWindowsMessage(self,webview,hwnd,msg,wParam,lParam,result):

        return self.mb.wkeFireWindowsMessage(webview,hwnd,msg,wParam,lParam,result)
    
    @staticmethod
    def fire_mouse_msg(hwnd,msg,x,y):
        pos=c_long(c_ushort(x).value | c_ulong(c_ushort(y).value).value << 16).value
        return user32.PostMessageW(hwnd,msg,0,pos)
    
    @staticmethod
    def fire_keyboard_msg(hwnd,virtualKeyCode,msg):
        return user32.SendMessageW(hwnd,msg,virtualKeyCode,0)

    @staticmethod
    def get_mouse_pos():
        p=mPos()
        user32.GetCursorPos(byref(p))

        return p



# -*- coding:utf-8 -*-
from .winConst import *
from .myctypes import *
from .wkeStruct import wkeProxy

class Proxy():
    def __init__(self,miniblink):     

        self.mb=miniblink
  
    def wkeSetProxy(self,ip,port,proxy_type=1,user=None,password=None):

        if not all([ip,port]):return
        if user==None:
            user=b''
        else:
            user=user.encode('utf8')
        if password==None:
            password=b''
        else:
            password=password.encode('utf8')
        ip=ip.encode('utf8')
        port=int(port)
        proxy= wkeProxy(type=c_int(proxy_type), hostname=ip, port=c_ushort(port),username=user,password=password)
        self.mb.wkeSetProxy(proxy)
    
    def wkeSetViewProxy(self,webview,ip,port,proxy_type=1,user=None,password=None):
        
        if not all([ip,port]):return
        if user==None:
            user=b''
        else:
            user=user.encode('utf8')
        if password==None:
            password=b''
        else:
            password=password.encode('utf8')
        ip=ip.encode('utf8')
        port=int(port)
        proxy= wkeProxy(type=c_int(proxy_type), hostname=ip, port=c_ushort(port),username=user,password=password)
        self.mb.wkeSetViewProxy(webview,proxy)


# -*- coding:utf-8 -*-
from .myctypes import user32
from .winConst import WinConst
import win32gui, win32ui
from struct import pack
import collections
import zlib



def to_png(data, size, level=6, file_name=None):
    width, height = size
    line = width * 3
    png_filter = pack(">B", 0)
    scanlines = b"".join(
        [png_filter + data[y * line : y * line + line] for y in range(height)]
    )

    magic = pack(">8B", 137, 80, 78, 71, 13, 10, 26, 10)
    
 
    ihdr = [b"", b"IHDR", b"", b""]
    ihdr[2] = pack(">2I5B", width, height, 8, 2, 0, 0, 0)
    ihdr[3] = pack(">I", zlib.crc32(b"".join(ihdr[1:3])) & 0xFFFFFFFF)
    ihdr[0] = pack(">I", len(ihdr[2]))

  
    idat = [b"", b"IDAT", zlib.compress(scanlines, level), b""]
    idat[3] = pack(">I", zlib.crc32(b"".join(idat[1:3])) & 0xFFFFFFFF)
    idat[0] = pack(">I", len(idat[2]))

    
    iend = [b"", b"IEND", b"", b""]
    iend[3] = pack(">I", zlib.crc32(iend[1]) & 0xFFFFFFFF)
    iend[0] = pack(">I", len(iend[2]))

    if not file_name:
        return magic + b"".join(ihdr + idat + iend)
    with open(file_name, "wb") as f:
        f.write(magic+ b"".join(ihdr + idat + iend))
    return None
class pixelsParse:
    __slots__ = {"__rgb", "pos", "raw", "size"}
    def __init__(self, data, left=0,top=0,width=0,height=0, size=None):

        _Pos = collections.namedtuple("Pos", "left, top")
        _Size = collections.namedtuple("Size", "width, height")
        self.__rgb = None  
        self.raw = data
        self.pos = _Pos(left, top)
        if size is not None:
            self.size = size
        else:
            self.size = _Size(width, height)
    @property
    def height(self):
        return self.size.height
    @property
    def left(self):
        return self.pos.left
    @property
    def rgb(self):
        
        if not self.__rgb:
            rgb = bytearray(self.height * self.width * 3)
            raw = self.raw
            rgb[0::3] = raw[2::4]
            rgb[1::3] = raw[1::4]
            rgb[2::3] = raw[0::4]
            self.__rgb = bytes(rgb)
        return self.__rgb
    @property
    def top(self):
        return self.pos.top
    @property
    def width(self):
        return self.size.width
def generate_bitBmp(hwnd=0,screenDC=0,left=0,top=0,width=0,height=0,cx=0,cy=0):

    if screenDC==0:

        screenDC = user32.GetWindowDC(hwnd)

    mfcDC = win32ui.CreateDCFromHandle(screenDC)

    saveDC = mfcDC.CreateCompatibleDC()

    saveBitMap = win32ui.CreateBitmap()

    saveBitMap.CreateCompatibleBitmap(mfcDC,width,height)
    
    saveDC.SelectObject(saveBitMap)
    
    saveDC.BitBlt((0,0), (width,height), mfcDC, (0, 0), WinConst.SRCCOPY)
    
    pixels = saveBitMap.GetBitmapBits(True)
    img=pixelsParse(data=bytearray(pixels), left=left
    ,top=top,width=width,height=height)
    pixels=None
    return img

def screen_shot(file_name,hwnd=0,screenDC=0,left=0,top=0,width=0,height=0):
    if not all([width,height]):return False
    img=generate_bitBmp(hwnd=hwnd, screenDC=screenDC,left=left,top=top,width=width,height=height,cx=0,cy=0)
    to_png(img.rgb, img.size, level=6, file_name=file_name)
    img=None
    return True
def whole_screenshot(mb,webview,file_name):

    left=0
    top=0
    width=mb.wkeGetContentWidth(webview)
    height=mb.wkeGetContentHeight(webview)
    pixels=create_string_buffer(width*height*4)
    mb.wkeResize(webview,width,height)
    mb.wkeUpdate()
    mb.wkePaint(webview,byref(pixels),0)
    img=pixelsParse(data=bytearray(pixels), left=left
        ,top=top,width=width,height=height)
    to_png(img.rgb, img.size, level=6, file_name=file_name)
    pixels=None
    img=None
    return True        




# -*- coding:utf-8 -*-
class WinConst():
    GWL_EXSTYLE = -20
    GWL_WNDPROC = -4
    WS_EX_LAYERED = 524288
    WM_PAINT = 15
    WM_ERASEBKGND = 20
    WM_SIZE = 5
    WM_KEYDOWN = 256
    WM_KEYUP = 257
    WM_CHAR = 258
    WM_LBUTTONDOWN = 513
    WM_LBUTTONUP = 514
    WM_MBUTTONDOWN = 519
    WM_RBUTTONDOWN = 516
    WM_LBUTTONDBLCLK = 515
    WM_MBUTTONDBLCLK = 521
    WM_RBUTTONDBLCLK = 518
    WM_MBUTTONUP = 520
    WM_RBUTTONUP = 517
    WM_MOUSEMOVE = 512
    WM_CONTEXTMENU = 123
    WM_MOUSEWHEEL = 522
    WM_SETFOCUS = 7
    WM_KILLFOCUS = 8
    WM_IME_STARTCOMPOSITION = 269
    WM_NCHITTEST = 132
    WM_GETMINMAXINFO = 36
    WM_DESTROY = 2
    WM_SETCURSOR = 32
    MK_CONTROL = 8
    MK_SHIFT = 4
    MK_LBUTTON = 1
    MK_MBUTTON = 16
    MK_RBUTTON = 2
    KF_REPEAT = 16384
    KF_EXTENDED = 256
    SRCCOPY = 13369376
    CAPTUREBLT = 1073741824
    CFS_POINT = 2
    CFS_FORCE_POSITION = 32
    OBJ_BITMAP = 7
    AC_SRC_OVER = 0
    AC_SRC_ALPHA = 1
    ULW_ALPHA = 2
    WM_INPUTLANGCHANGE = 81
    WM_NCDESTROY = 130
    IMAGE_ICON=1
    LR_LOADFROMFILE=16
    WM_SETICON=128
    ICON_SMALL=0
    ICON_BIG=1
    IMAGE_ICON = 1
    LR_CREATEDIBSECTION = 0x00002000
    SRCCOPY = 13369376


# -*- coding:utf-8 -*-

from .winConst import *
from .myctypes import *
from .method import method
from config import _LRESULT
class JsRunPy():
    def __init__(self,miniblink):

        self.mb=miniblink

    def get_js_args_val(self,es,arg_count):
        val_ls=[None]*arg_count
        for i in range(arg_count):

            arg_type=self.mb.jsArgType(es,i)
            
            if arg_type==0:
                val=self.mb.jsArg(es,i)
                val=self.mb.jsToInt(es, c_longlong(val))
            elif arg_type==1:
                val=self.mb.jsArg(es,i)
                val=self.mb.jsToTempStringW(es, c_longlong(val))            
            elif arg_type==2:
                val=self.mb.jsArg(es,i)
                val=self.mb.jsToTempStringW(es, c_longlong(val))
                if val=='false':
                    val=False
                else:
                    val=True
           
            elif arg_type==5 or arg_type==7:
                val=None
            elif arg_type==6:
         
                val=self.mb.jsArg(es,i)
                lens=self.mb.jsGetLength(es,c_longlong(val))
                tmp_arr=[None]*lens
                for j in range(lens):
                    tmp_val=self.mb.jsGetAt(es,c_longlong(val),j)            
                    if self.mb.jsIsNumber(tmp_val)==1:
                        tmp_val=self.mb.jsToInt(es, c_longlong(tmp_val))
                    elif self.mb.jsIsString(tmp_val)==1:
                        tmp_val=self.mb.jsToTempStringW(es, c_longlong(tmp_val))
                        tmp_val=c_wchar_p(tmp_val).value
                    elif self.mb.jsIsBoolean(tmp_val)==1:
                        tmp_val=self.mb.jsToTempStringW(es, c_longlong(tmp_val))
                        if tmp_val=='false':
                            tmp_val=False
                        else:
                            tmp_val=True
                    tmp_arr[j]=tmp_val
                val=tmp_arr
            else:
                val=self.mb.jsArg(es,i)
                val=self.mb.jsToTempStringW(es, c_longlong(val))
            val_ls[i]=val
      
        return val_ls
  
    def to_js_args_val(self,es,val):

        if isinstance(val,str):
            val=self.mb.jsStringW(es,val)
        elif isinstance(val,int):
            val=self.mb.jsInt(val)
        elif isinstance(val,float):
            val=self.mb.jsFloat(val)
        elif isinstance(val,bool):
            val=self.mb.jsBoolean(val)

        elif isinstance(val,list):
            lens=len(val)
            tmp_arr=self.mb.jsEmptyArray(es)
            for i in range(lens):
                if isinstance(val[i],int):
                    tmp_val=self.mb.jsInt(val[i])
                elif isinstance(val[i],str):
                    tmp_val=self.mb.jsStringW(es,val[i])
                elif isinstance(val[i],float):
                    tmp_val=self.mb.jsFloat(c_float(val[i]))
                self.mb.jsSetAt(es, c_longlong(tmp_arr), i, c_longlong(tmp_val))
            val=tmp_arr
        elif isinstance(val,dict): 
            tmp_obj=self.mb.jsEmptyObject(es)
            for k,v in val.items():
                if isinstance(v,int):
                    v=self.mb.jsInt(v)
                elif isinstance(v,str):
                    v=self.mb.jsStringW(es,v)
                elif isinstance(v,float):
                    v=self.mb.jsFloat(c_float(v))
                self.mb.jsSet(es,c_longlong(tmp_obj),k.encode(),c_longlong(v))
            val=tmp_obj
        return val
 
    def bind_func(self,func_name,arg_count=0,param=0):
        js_bind_func=getattr(self,func_name)
        func_name=func_name.encode()
        self.mb.wkeJsBindFunction(func_name,js_bind_func,param,arg_count)   

    @method(CFUNCTYPE(c_longlong, _LRESULT, c_void_p))
    def call_py_func(self,es,param):

        if hasattr(self,'python_func'):
            return self.python_func(es=es,param=param)
        return 0


# -*- coding:utf-8 -*-
from .winConst import *
from .myctypes import *
from .method import method


class PyRunJS():
    def __init__(self,miniblink):

        self.mb=miniblink
  
    def run_js(self,webview,js_code):

        es=self.mb.wkeGlobalExec(webview)
        val=self.mb.wkeRunJSW(webview,js_code)
        val=self.mb.jsToStringW(es,val)
        if val=='undefined':
            val=None
        # print(val,'run_js')
        return val
  
    def run_js_file(self,webview,file_name):
        with open(file_name) as f:
            js_code=f.read()
            return self.run_js(webview,js_code)        
    def run_js_byframe(self,webview,frameId,js_code,isInClosure=True):
        
        js_code=js_code.encode()
        val=self.mb.wkeRunJsByFrame(webview,frameId,js_code,isInClosure)
        es = self.mb.wkeGetGlobalExecByFrame(webview, frameId)
        val=self.mb.jsToTempStringW(es, c_longlong(val))
        # print(val,'run_js_byframe',frameId)
        return val
 
    def run_js_global(self,webview,func_name,param_ls=[],this_func=0):
       
        es=self.mb.wkeGlobalExec(webview)
        if this_func==0:
            func_name=func_name.encode()
            func=self.mb.jsGetGlobal(es,func_name)
        else:
            ...

        argCount=len(param_ls)
        args_ls=(c_longlong *argCount)()

        for i,param in enumerate(param_ls):
            if isinstance(param,str):
                param=self.mb.jsStringW(es,param)
            elif isinstance(param,int):
                param=self.mb.jsInt(c_longlong(param))
            elif isinstance(param,float):
                param=self.mb.jsFloat(param)
            elif isinstance(param,bool):
                param=self.mb.jsBoolean(param)
            args_ls[i]=param

        callRet=self.mb.jsCall(es,c_longlong(func),c_longlong(this_func),byref(args_ls),c_longlong(argCount))
        val=self.mb.jsToStringW(es,c_longlong(callRet))
        # print(val,'run_js_global')
        return val        


# -*- coding:utf-8 -*-

from .pyrunjs import PyRunJS
class Cookie():
    def __init__(self,miniblink):        
        self.mb=miniblink
        self.js=PyRunJS(miniblink)
    def wkeGetCookieW(self,webview):

        return self.mb.wkeGetCookieW(webview)

    def wkeSetCookie(self,webview,url,cookie):

        cookie=cookie.split(';')
        for x in cookie:
            self.mb.wkeSetCookie(webview,url.encode('utf8'),x.encode('utf8'))
 
        self.mb.wkePerformCookieCommand(webview,2)

    def set_cookie_by_js(self,webview,url,cookie):
       
        js_code=f"var cookie='{cookie}';"+'''
        cookie.split(';').forEach(function(e){
        document.cookie=e
        })'''
        self.js.run_js(webview,js_code)
       
        self.mb.wkeLoadURLW(webview, url)
 
    def wkeSetCookieJarPath(self,webview,path):
        
        self.mb.wkeSetCookieJarPath(webview,path)
   
    def wkeSetCookieJarFullPath(self,webview,path):
      
        self.mb.wkeSetCookieJarFullPath(webview,path)
    
    def wkeClearCookie(self,webview):

        self.mb.wkeClearCookie(webview)


# -*- coding:utf-8 -*-
from .winConst import *
from .myctypes import *
from .wkeStruct import (wkeRect,wkeWindowFeatures,wkeMemBuf)
from .method import method
from .defaultcallbackfunc import destroy_func
from config import _LRESULT

class CallBack():
    def __init__(self,miniblink,width=360,height=480):
        self.mb=miniblink  
        self.width=width
        self.height=height
        self.wkeWindowDestroyCallback=destroy_func

    def wkeOnCreateView(self,webview,param=0):
        self.mb.wkeOnCreateView(webview,self._wkeCreateViewCallback,param)
    def wkeOnWindowClosing(self,webview,param=0):
        self.mb.wkeOnWindowClosing(webview,self._wkeWindowClosingCallback,param)
    def wkeOnWindowDestroy(self,webview,param=0):
        self.mb.wkeOnWindowDestroy(webview,self._wkeWindowDestroyCallback,param)
    def wkeOnPaintUpdated(self,webview,param=0):
        self.mb.wkeOnPaintUpdated(webview,self._wkePaintUpdatedCallback ,param)
    def wkeOnPaintBitUpdated(self,webview,param=0):
        self.mb.wkeOnPaintBitUpdated(webview, self._wkePaintBitUpdatedCallback,param)
    def wkeOnNavigation(self,webview,param=0):
        self.mb.wkeOnNavigation(webview,self._wkeNavigationCallback,param)
    def wkeOnTitleChanged(self,webview,param=0):
        self.mb.wkeOnTitleChanged(webview,self._wkeTitleChangedCallback,param)
    def wkeOnURLChanged2(self,webview,param=0):
        self.mb.wkeOnURLChanged2(webview,self._wkeURLChangedCallback2,param)
    def wkeOnMouseOverUrlChanged(self,webview,param=0):
        self.mb.wkeOnMouseOverUrlChanged(webview,self._wkeMouseOverUrlChangedCallback,param)
    def wkeOnAlertBox(self,webview,param=0):
        self.mb.wkeOnAlertBox(webview,self._wkeAlertBoxCallback,param)
    def wkeOnConfirmBox(self,webview,param=0):
        self.mb.wkeOnConfirmBox(webview,self._wkeConfirmBoxCallback,param)
    def wkeOnPromptBox(self,webview,param=0):
        self.mb.wkeOnPromptBox(webview,self._wkePromptBoxCallback,param)
    def wkeOnConsole(self,webview,param=0):
        self.mb.wkeOnConsole(webview,self._wkeConsoleCallback,param)
    def wkeOnDownload(self,webview,param=0):
        self.mb.wkeOnDownload(webview,self._wkeDownloadCallback,param)
    def wkeOnDocumentReady2(self,webview,param=0):
        self.mb.wkeOnDocumentReady2(webview,self._wkeDocumentReady2Callback,param)
    def wkeNetOnResponse(self,webview,param=0):
        self.mb.wkeNetOnResponse(webview,self._wkeNetResponseCallback,param)
    def wkeOnLoadUrlBegin(self,webview,param=0):
        self.mb.wkeOnLoadUrlBegin(webview,self._wkeLoadUrlBeginCallback,param)
    def wkeOnLoadUrlEnd(self,webview,param=0):
        self.mb.wkeOnLoadUrlEnd(webview,self._wkeLoadUrlEndCallback,param)
    def wkeOnLoadUrlFail(self,webview,param=0):
        self.mb.wkeOnLoadUrlFail(webview,self._wkeLoadUrlFailCallback,param)
    def wkeOnLoadingFinish(self,webview,param=0):
        self.mb.wkeOnLoadingFinish(webview,self._wkeLoadingFinishCallback,param)
    def wkeNetGetFavicon(self,webview,param=0):
        self.mb.wkeNetGetFavicon(webview,self._wkeOnNetGetFaviconCallback,param)

    @method(CFUNCTYPE(c_bool, _LRESULT, c_void_p))
    def _wkeWindowClosingCallback(self,webview, param):
        if hasattr(self,'wkeWindowClosingCallback'):
            return self.wkeWindowClosingCallback(webview=webview,param=param)
        return True
    @method(CFUNCTYPE(None, _LRESULT, c_void_p))
    def _wkeWindowDestroyCallback(self,webview, param):
        if hasattr(self,'wkeWindowDestroyCallback'):
            self.wkeWindowDestroyCallback(webview=webview,param=param)
    @method(CFUNCTYPE(None,_LRESULT,c_void_p,_LRESULT,c_int,c_int,c_int,c_int))
    def _wkePaintUpdatedCallback(self,webview,param,hdc,x,y,cx,cy):
        if hasattr(self,'wkePaintUpdatedCallback'):
            return self.wkePaintUpdatedCallback(webview=webview,param=param,hdc=hdc,x=x,y=y,cx=cx,cy=cy)
    @method(CFUNCTYPE(None,_LRESULT,c_void_p,_LRESULT,POINTER(wkeRect),c_int,c_int))
    def _wkePaintBitUpdatedCallback(self,webview,param,buf,rect,width,height):
        if hasattr(self,'wkePaintBitUpdatedCallback'):
            return self.wkePaintBitUpdatedCallback(webview=webview,param=param,buf=buf,rect=rect,width=width,height=height)  
    @method(CFUNCTYPE(None,_LRESULT,c_void_p,c_void_p))
    def _wkeDocumentReady2Callback(self,webview,param,frameId):
        if hasattr(self,'wkeDocumentReady2Callback'):
            return self.wkeDocumentReady2Callback(webview=webview,param=param,frameId=frameId)
    @method(CFUNCTYPE(_LRESULT, _LRESULT, c_void_p,c_int,c_void_p,POINTER(wkeWindowFeatures)))
    def _wkeCreateViewCallback(self,webview,param,navigationType,url,windowFeatures):

        if hasattr(self,'wkeCreateViewCallback'):
            return self.wkeCreateViewCallback(webview=webview,param=param,navigationType=navigationType,url=url,windowFeatures=windowFeatures)
        return 0
    @method(CFUNCTYPE(None, _LRESULT, c_void_p,c_int,c_void_p))
    def _wkeNavigationCallback(self,webview,param,navigationType,url):

        if hasattr(self,'wkeNavigationCallback'):
            return self.wkeNavigationCallback(webview=webview,param=param,navigationType=navigationType,url=url)
    @method(CFUNCTYPE(None, _LRESULT, c_void_p, c_void_p))
    def _wkeTitleChangedCallback(self,webview, param, title):
        if hasattr(self,'wkeTitleChangedCallback'):
            return self.wkeTitleChangedCallback(webview=webview, param=param, title=title)
    @method(CFUNCTYPE(None, _LRESULT, c_void_p, c_void_p))
    def _wkeMouseOverUrlChangedCallback(self,webview, param, url):
 
        if hasattr(self,'wkeMouseOverUrlChangedCallback'):
            return self.wkeMouseOverUrlChangedCallback(webview=webview, param=param, url=url)
    @method(CFUNCTYPE(None, _LRESULT, c_void_p,c_int,c_void_p))
    def _wkeURLChangedCallback2(self,webview,param,frameId,url):
        if hasattr(self,'wkeURLChangedCallback2'):
            return self.wkeURLChangedCallback2(webview=webview, param=param,frameId=frameId,url=url)
    @method(CFUNCTYPE(None, _LRESULT, c_void_p,c_void_p))
    def _wkeAlertBoxCallback(self,webview,param,msg):

        if hasattr(self,'wkeAlertBoxCallback'):
            return self.wkeAlertBoxCallback(webview=webview, param=param,msg=msg)

    @method(CFUNCTYPE(c_bool, _LRESULT, c_void_p,c_void_p))
    def _wkeConfirmBoxCallback(self,webview,param,msg):

        if hasattr(self,'wkeConfirmBoxCallback'):
            return self.wkeConfirmBoxCallback(webview=webview, param=param,msg=msg)
        return False
    @method(CFUNCTYPE(c_bool, _LRESULT, c_void_p,c_void_p,c_void_p,c_void_p))
    def _wkePromptBoxCallback(self,webview,param,msg,defaultResult,result):

        if hasattr(self,'wkePromptBoxCallback'):
            return self.wkePromptBoxCallback(webview=webview, param=param,msg=msg,defaultResult=defaultResult,result=result)
        return False
    @method(CFUNCTYPE(None, _LRESULT,c_void_p, c_int,c_void_p,c_void_p,c_ulonglong,c_void_p))
    def _wkeConsoleCallback(self,webview,param,level,msg,sourceName,sourceLine,stackTrace):

        if hasattr(self,'wkeConsoleCallback'):
            return self.wkeConsoleCallback(webview=webview, param=param,level=level,msg=msg,sourceName=sourceName,sourceLine=sourceLine,stackTrace=stackTrace)
    @method(CFUNCTYPE(c_bool,_LRESULT,c_void_p,c_char_p))
    def _wkeDownloadCallback(self,webview,param,url):

        if hasattr(self,'wkeDownloadCallback'):
            self.wkeDownloadCallback(webview=webview, param=param,url=url)
        return True
    @method(CFUNCTYPE(c_bool,_LRESULT,c_void_p,c_char_p,c_void_p))
    def _wkeNetResponseCallback(self,webview,param,url,job):
    
        if hasattr(self,'wkeNetResponseCallback'):
            return self.wkeNetResponseCallback(webview=webview, param=param,url=url,job=job)
        return False
    @method(CFUNCTYPE(c_bool,_LRESULT,c_void_p,c_char_p,c_void_p))
    def _wkeLoadUrlBeginCallback(self,webview,param,url,job):

        if hasattr(self,'wkeLoadUrlBeginCallback'):
            return self.wkeLoadUrlBeginCallback(webview=webview, param=param,url=url,job=job)
        return False
    @method(CFUNCTYPE(None,_LRESULT,c_void_p,c_char_p,c_void_p,c_void_p,c_int))
    def _wkeLoadUrlEndCallback(self,webview,param,url,job,buf,lens):
    
        if hasattr(self,'wkeLoadUrlEndCallback'):
            return self.wkeLoadUrlEndCallback(webview=webview, param=param,url=url,job=job,buf=buf,lens=lens)
    @method(CFUNCTYPE(None,_LRESULT,c_void_p,c_char_p,c_void_p))
    def _wkeLoadUrlFailCallback(self,webview,param,url,job):

        if hasattr(self,'wkeLoadUrlFailCallback'):
            return self.wkeLoadUrlFailCallback(webview=webview, param=param,url=url,job=job)
    @method(CFUNCTYPE(None,_LRESULT,c_void_p,c_void_p,c_int,c_void_p))
    def _wkeLoadingFinishCallback(self,webview,param,url,result,failedReason):
 
        if hasattr(self,'wkeLoadingFinishCallback'):
            return self.wkeLoadingFinishCallback(webview=webview, param=param,url=url,result=result,failedReason=failedReason)
    @method(CFUNCTYPE(None, _LRESULT, c_void_p,c_char_p,POINTER(wkeMemBuf)))
    def _wkeOnNetGetFaviconCallback(self,webview,param,url,buf):
  
        if hasattr(self,'wkeOnNetGetFaviconCallback'):
            return self.wkeOnNetGetFaviconCallback(webview=webview, param=param,url=url,buf=buf)
    

    @method(WINFUNCTYPE(None,HWND, UINT,UINT,DWORD))
    def _timerProc(self,hwnd,msg,nid,dwTime):
        return self.timerProc(hwnd=hwnd,msg=msg,nid=nid,dwTime=dwTime)



# -*- coding:utf-8 -*-
from .winConst import *
from .myctypes import *
from .wkeStruct import Rect
class Window():
    def __init__(self,miniblink):
        self.width=360
        self.height=480
        self.mb=miniblink
    def wkeCreateWebWindow(self,_type=0,x=0,y=0,width=360,height=480,hwnd=0):
        
        self.mb.wkeCreateWebWindow.restype=c_longlong
        webview =self.mb.wkeCreateWebWindow(
            _type,hwnd, x, y,
            width, height
        )
        return webview
    def wkeShowWindow(self,webview,_bool=True):
        self.mb.wkeShowWindow(webview,_bool)
    def wkeCreateWebView(self):
   
        return self.mb.wkeCreateWebView()
    def wkeDestroyWebView(self,webview):
        self.mb.wkeDestroyWebView(webview)
    def wkeMoveToCenter(self,webview):
        self.mb.wkeMoveToCenter(webview)
    def wkeRunMessageLoop(self):
        self.mb.wkeRunMessageLoop(0)
    def wkeSetUserAgentW(self,webview,ua):
        self.mb.wkeSetUserAgentW(webview,ua)
    def wkeGetUserAgent(self,webview):
        ua=self.mb.wkeGetUserAgent(webview)
        return ua.decode()        
    def wkeSetDragEnable(self,webview,_bool):
   
        self.mb.wkeSetDragEnable(webview,_bool)
    def wkeAddPluginDirectory(self,webview,_path):
 
        self.mb.wkeAddPluginDirectory(webview,_path)
    def wkeSetNpapiPluginsEnabled(self,webview,_bool,_path=None):
      
        if _path!=None:
            self.wkeAddPluginDirectory(webview,_path)
        self.mb.wkeSetNpapiPluginsEnabled(webview,_bool)
    def wkeSetCspCheckEnable(self,webview,_bool=False):

        self.mb.wkeSetCspCheckEnable(webview,_bool)
    def wkeSetDebugConfig(self,webview,debug,param):
       
        debug=debug.encode()
        if isinstance(param,str):
            param=param.encode()
        self.mb.wkeSetDebugConfig(webview,debug,param)
    def wkeSetHeadlessEnabled(self,webview,_bool):
      
        self.mb.wkeSetHeadlessEnabled(webview,_bool)
    def wkeSetNavigationToNewWindowEnable(self,webview,_bool):
       
        self.mb.wkeSetNavigationToNewWindowEnable(webview,_bool)
    def wkeSetWebViewName(self,webview,name):
        self.mb.wkeSetWebViewName(webview,name.encode())
    def wkeSetZoomFactor(self,webview,factor):
 
        self.mb.wkeSetZoomFactor(webview,c_float(factor))
    def wkeSetContextMenuEnabled(self,webview,_bool):
        
        self.mb.wkeSetContextMenuEnabled(webview,_bool)
    def wkeSetWindowTitleW(self,webview,title):
        self.mb.wkeSetWindowTitleW(webview, title)
    def wkeSetTransparent(self,webview,_bool):
  
        self.mb.wkeSetTransparent(webview,_bool)
    def wkeSetHandleOffset(self,webview,x,y):
      
        self.mb.wkeSetHandleOffset(webview,x,y)
    def wkeSetHandle(self,webview,hwnd):

        self.mb.wkeSetHandle(webview,hwnd)
    def wkeResize(self,webview,width=0,height=0):
        
        if width==0 or height==0:return False
        self.mb.wkeResize(webview,width,height)
        return True
    def wkeWidth(self,webview):
  
        return self.mb.wkeWidth(webview)
    def wkeHeight(self,webview):
     
        return self.mb.wkeHeight(webview)
    def wkeContentsWidth(self,webview):
      
        return self.mb.wkeContentsWidth(webview)
    def wkeContentsHeight(self,webview):
       
        return self.mb.wkeContentsHeight(webview)
    def wkeGoForward(self,webview):
        self.mb.wkeGoForward(webview)
    def wkeGoBack(self,webview):
        self.mb.wkeGoBack(webview)
    def wkeGetWindowHandle(self,webview):
     
        return self.mb.wkeGetWindowHandle(webview)

    def wkeGetViewDC(self,webview):
     
        hdc=self.mb.wkeGetViewDC(webview)
        return hdc
    def wkeSetTouchEnabled(self,webview,_bool):
   
        b=not _bool
        self.mb.wkeSetTouchEnabled(webview,_bool)
      
        self.mb.wkeSetMouseEnabled(webview,b)
    def simulate_device(self,webview,key,value,_int,_float):
   
        key=key.encode()
        if value=='':
            value=b''
        else:
            value=value.encode()
        self.mb.wkeSetDeviceParameter(webview, key,value,_int,c_float(_float))
    def bind_window(self,hwnd=0,show=True):
   
        if hwnd==0:
            user32.PostQuitMessage(0)
            return 0
        webview=self.wkeCreateWebWindow(_type=2,hwnd=hwnd)
        rc=Window.get_client_rect(hwnd)
        width = rc.Right - rc.Left
        height = rc.Bottom - rc.Top
 
        self.wkeResize(webview,width,height)
        self.wkeShowWindow(webview,show)
        return webview
    def message_loop(self,webview=None):
    
        msg = MSG()
        while user32.GetMessageW(byref(msg), None, 0, 0) > 0:
            user32.TranslateMessage(byref(msg))
            user32.DispatchMessageW(byref(msg))


    @staticmethod
    def get_window_rect(hwnd):
        rect=Rect()
        user32.GetWindowRect(hwnd,byref(rect))
        return rect

    @staticmethod
    def get_client_rect(hwnd):
        rect=Rect()
        user32.GetClientRect(hwnd,byref(rect))
        return rect



# -*- coding:utf-8 -*-
from .myctypes import user32,kernel32
from .winConst import WinConst
from os import path

RelPath = lambda file : path.join(path.dirname(path.abspath(__file__)), file)

def set_icon(hwnd, filename):
    if not filename.endswith('.ico'):
        return False
    hModel = kernel32.GetModuleHandleW(None)
    hIcon = user32.LoadImageW(hModel,RelPath(filename),WinConst.IMAGE_ICON,48, 48,WinConst.LR_LOADFROMFILE | WinConst.LR_CREATEDIBSECTION)
    user32.SendMessageW(hwnd, WinConst.WM_SETICON, WinConst.ICON_BIG, hIcon)
    return True
def set_icon_2(hwnd,filename):
    if not filename.endswith('.ico'):
        return False
    import win32api
    import win32gui
    icon_big = win32gui.LoadImage(
        None, RelPath(filename), WinConst.IMAGE_ICON,
        48, 48, WinConst.LR_LOADFROMFILE)
    win32api.SendMessage(hwnd, WinConst.WM_SETICON,WinConst.ICON_BIG, icon_big)
    return True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mbpython']

package_data = \
{'': ['*']}

install_requires = \
['pywin32>=228,<229']

setup_kwargs = {
    'name': 'mbpython',
    'version': '0.1.4',
    'description': 'Miniblink binding for python',
    'long_description': '# MBPython\n\nMiniblink is a lighter browser widget base on chromium,\njust one file, faster browser kernel of blink to integrate HTML UI in your app.\n\nMBPython is an open source project founded by lochen to provide python bindings for the Miniblink. Examples of embedding Miniblink browser are available for many popular GUI toolkits including: wxPython, PyQt, PySide, Kivy, Panda3D, PyGTK, PyGObject, PyGame/PyOpenGL and PyWin32.\n\nBy using C interface, you can create a browser just some line code.\nThere are many use cases for MBPython. You can embed a web browser control based on MBPython with great HTML 5 support. You can use it to create a HTML 5 based GUI in an application, this can act as a replacement for standard GUI toolkits like wxWidgets, Qt or GTK. You can render web content off-screen in application that use custom drawing frameworks. You can use it for automated testing of existing applications. You can use it for web scraping or as a web crawler, or other kind of internet bots.\n\n### Latest release\n\nOS | Py3 | 32bit | 64bit | Requirements\n--- | --- | --- | --- | ---\nWindows | 3.6+ | Yes | Yes | Windows 7+\n\n### Installation\n\n```bash\npip install --user MBPython\n```\n\n### How to use\nMust create a new file named config.py like this demo → https://github.com/lochen88/MBPython/blob/master/tests/config.py\nand download the node.dll add to your project.\n\n<p style="color: red">\n    The free version can only be called on the UI thread.\n</p>\n\nCreate a simple Window\n```bash\nfrom MBPython import miniblink\nwke=miniblink.MiniBlink\nmb=wke.init()\nwindow=wke.window\nwebview=window.wkeCreateWebWindow(0,0,0,860,760)\nwindow.wkeShowWindow(webview)\nwindow.wkeRunMessageLoop()\n```\n\n### PyInstaller\n\n```bash\npyinstall xxx.py --noconsole\n```\n\n### Examples\n\nSee the [tests](https://github.com/lochen88/MBPython/tree/master/tests) folder\n\n### Tutorial\n\nSee the [documents](https://github.com/lochen88/MBPython/tree/master/documents) folder\n\n### What about CEF?\n\nIn short: I do not like CEF, it is too big, comes with too many dependency resolution library, and I think we can make a better and more intuitive one. Here are a few things that I don\'t like.\n\n### Resources\n\n<table border="0">\n    <tr>\n        <td width="50%" valign="top">\n            <p align="center">\n                <img src="https://s1.ax1x.com/2020/10/15/0onOBV.jpg">\n            </p>\n            if you have any questions,you can contact me,and i will try my best to help you\n        </td>\n        <td width="50%" valign="top">\n            <p align="center">\n                <img src="https://s1.ax1x.com/2020/10/15/0onIhQ.jpg">\n            </p>\n            If you would like to support general MBPython development efforts by making a donation then please scan to pay by the alipay\n        </td>\n    </tr>\n</table>\n\n* [Project Website](https://github.com/lochen88/MBPython)\n* [Issue Tracker](https://github.com/lochen88/MBPython/issues)\n* [Official Website](https://weolar.github.io/miniblink/)\n* [QQ Group-198671899](https://qm.qq.com/cgi-bin/qm/qr?k=JipVq9gRdpPf4dqjPK9bkL99u-_BLwZz&jump_from=webapi)\n\n\n',
    'author': 'lochen',
    'author_email': '1191826896@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/lochen88/MBPython',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# -*- coding:utf-8 -*-
from ctypes import (
    c_int,
    c_short,
    c_ushort,
    c_ulong,
    c_long,
    c_longlong,
    c_ulonglong,
    c_float,
    c_char,
    c_char_p,
    c_wchar_p,
    c_bool,
    c_void_p,
    c_size_t,
    Structure,
    byref,
    POINTER,
    create_string_buffer,
    sizeof,
    windll,
    cdll,
    CFUNCTYPE,
    WINFUNCTYPE,
    addressof
    )

from ctypes.wintypes import (
    DWORD,
    HWND,
    INT,
    LONG,
    LPARAM,
    UINT,
    WORD,
    WPARAM,
    RGB,
    MSG,
    HMODULE,
    LPCWSTR,
    HINSTANCE,
    HANDLE,
    LPVOID,
    BYTE
)
user32 = windll.user32
gdi32 = windll.gdi32
imm32= windll.imm32
kernel32= windll.kernel32




# -*- coding:utf-8 -*-
def method(prototype):
    class MethodDescriptor(object):
        __slots__ = ['func', 'bound_funcs']
        def __init__(self, func):
            self.func = func
            self.bound_funcs = {} 
        def __get__(self, obj, type=None):
            if obj!=None:
                try:
                    return self.bound_funcs[obj,type]
                except:
                    ret = self.bound_funcs[obj,type] = prototype(
                        self.func.__get__(obj, type))
                    return ret
    return MethodDescriptor